"use strict";

import React, { memo } from 'react';
import { Pressable, View } from 'react-native';
import { Canvas, Shadow } from '@shopify/react-native-skia';
import Animated, { interpolateColor, useAnimatedReaction, useDerivedValue, withTiming } from 'react-native-reanimated';
import { COMMON_STYLES, DAMPING_DURATION, DAMPING_RATIO, INITIAL_DEPTH, REFLECTED_LIGHT_COLOR, SHADOW_BLUR, SHADOW_COLOR } from "../constants.js";
import { getBackgroundColor, getShadowProperty, isLinearProps, numerify } from "../utils.js";
import LinearGradientFill from "./ShadowLinearGradientFill.js";
import { useAnimatedOffset } from "../hooks/useAnimatedOffset.js";
import { CornerRadii } from "./CornerRadii.js";
import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
const PressButton = Animated.createAnimatedComponent(Pressable);
export const UnifiedShadowToggle = /*#__PURE__*/memo(function ShadowToggle({
  width,
  height,
  isActive = false,
  activeColor,
  shadowBlur = SHADOW_BLUR,
  shadowColor = SHADOW_COLOR,
  reflectedLightColor = REFLECTED_LIGHT_COLOR,
  duration = DAMPING_DURATION,
  damping = DAMPING_RATIO,
  isReflectedLightEnabled = true,
  shadowOffset,
  reflectedLightOffset: rfOffset,
  reflectedLightBlur,
  style,
  backgroundColor,
  children,
  ...props
}) {
  // Determine the final background color (pulling from `props.style` or a default).
  const _backgroundColor = getBackgroundColor({
    backgroundColor,
    style
  });
  const shadowProps = getShadowProperty({
    shadowOffset,
    shadowColor,
    shadowBlur,
    reflectedLightOffset: rfOffset,
    reflectedLightColor,
    reflectedLightBlur
  });
  const isLinear = isLinearProps(props);
  const styleWidth = width ?? numerify(style?.width, 0);
  const styleHeight = height ?? numerify(style?.height, 0);
  const [layoutSize, setLayoutSize] = React.useState({
    width: styleWidth,
    height: styleHeight
  });
  // Decide if we even need to attach onLayout; turns on when width or height is: undefined, NaN, or 0
  const needMeasure = !styleWidth || !styleHeight;

  // onLayout only does something if we truly need measure
  const onLayout = React.useCallback(e => {
    if (!needMeasure) return;
    const {
      width: w,
      height: h
    } = e.nativeEvent.layout;
    setLayoutSize({
      width: w,
      height: h
    });
  }, [needMeasure]);
  const {
    depth,
    offset,
    reflectedLightOffset,
    inset,
    blurRadius,
    PressedAnimatedStyle
  } = useAnimatedOffset({
    offset: shadowProps.shadowOffset,
    reflectedLightOffset: shadowProps.reflectedLightOffset,
    blurRadius: shadowProps.shadowBlur,
    damping,
    duration,
    onPressIn: props.onPressIn,
    onPressOut: props.onPressOut
  });

  // Only show the canvas if we have a valid size
  const canRenderCanvas = layoutSize.width > 0 && layoutSize.height > 0;
  const animatedBackgroundColor = useDerivedValue(() => interpolateColor(depth.value, [INITIAL_DEPTH, -INITIAL_DEPTH], [_backgroundColor, activeColor || _backgroundColor]));
  useAnimatedReaction(() => isActive, next => {
    depth.value = withTiming(next ? -INITIAL_DEPTH * damping : INITIAL_DEPTH, {
      duration
    });
  }, [isActive]);
  return /*#__PURE__*/_jsxs(View, {
    onLayout: onLayout,
    style: [COMMON_STYLES.canvasWrapper],
    children: [canRenderCanvas && /*#__PURE__*/_jsx(Canvas, {
      style: [COMMON_STYLES.canvas, {
        width: layoutSize.width * 1.4,
        height: layoutSize.height * 1.4
      }],
      children: /*#__PURE__*/_jsxs(CornerRadii, {
        width: layoutSize.width,
        height: layoutSize.height,
        style: style,
        backgroundColor: animatedBackgroundColor,
        children: [isLinear && /*#__PURE__*/_jsx(LinearGradientFill, {
          ...props,
          // from, to, colors, etc.
          width: layoutSize.width,
          height: layoutSize.height
        }), /*#__PURE__*/_jsx(Shadow, {
          dx: offset.dx,
          dy: offset.dy,
          blur: blurRadius,
          color: shadowColor,
          inner: inset
        }), isReflectedLightEnabled && /*#__PURE__*/_jsx(Shadow, {
          dx: reflectedLightOffset.dx,
          dy: reflectedLightOffset.dy,
          blur: blurRadius,
          color: reflectedLightColor,
          inner: true
        })]
      })
    }), /*#__PURE__*/_jsx(PressButton, {
      ...props,
      style: [style, COMMON_STYLES.canvasWrapper],
      children: /*#__PURE__*/_jsx(Animated.View, {
        style: PressedAnimatedStyle,
        children: children
      })
    })]
  });
});

/**
 * ShadowToggle
 * ----------------
 * A toggle component that casts a shadow when active.
 * The shadow effect is created using the `@shopify/react-native-skia` library.
 *
 * @param isActive - Whether the shadow is active
 * @param activeColor - The color of the shadow when active
 */
export const ShadowToggle = /*#__PURE__*/memo(function ShadowToggle({
  ...props
}) {
  return /*#__PURE__*/_jsx(UnifiedShadowToggle, {
    ...props
  });
});

/**
 * LinearShadowToggle
 * ----------------
 * A toggle component that casts a linear gradient shadow when active.
 * The shadow effect is created using the `@shopify/react-native-skia` library.
 *
 * @param isActive - Whether the shadow is active
 * @param activeColor - The color of the shadow when active
 * @param colors - The colors of the linear gradient
 * @param from - The direction of the linear gradient
 * @param to - The direction of the linear gradient
 */
export const LinearShadowToggle = /*#__PURE__*/memo(function LinearShadowToggle({
  ...props
}) {
  return /*#__PURE__*/_jsx(UnifiedShadowToggle, {
    ...props
  });
});
//# sourceMappingURL=ShadowToggle.js.map